==> adc-lib-v2.5.4.tar: adc-lib-v2.5.4/pytest/fmc-adc-100m14b4cha/test_adc_trigger.py <==
"""
SPDX-License-Identifier: GPL-3.0-or-later
SPDX-FileCopyrightText: 2020 CERN
"""

import pytest
from PyAdcLib import PyAdcConf, PyFmcAdc100m14b4ch, timeval, PyAdcAbstract

class TestAdcTriggerSoftware(object):

    def test_adc_trg_sw(self, adc_simple):
        """Test that the system triggers when we do the software trigger"""
        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_BRD, 0)
        conf.mask_set(PyAdcConf.ADC_CONF_BRD_STATE_MACHINE_STATUS)
        adc_simple.retrieve_config(conf)
        assert conf.value_get(PyAdcConf.ADC_CONF_BRD_STATE_MACHINE_STATUS) == adc_simple.ADC_CONF_100M14B4CHA_FSM_STATE_IDLE
        adc_simple.acq_start(PyFmcAdc100m14b4ch.ADC_F_FLUSH,
                                 timeval(0,0))
        adc_simple.retrieve_config(conf)
        assert conf.value_get(PyAdcConf.ADC_CONF_BRD_STATE_MACHINE_STATUS) == adc_simple.ADC_CONF_100M14B4CHA_FSM_STATE_WAIT

        adc_simple.trigger_fire()
        adc_simple.retrieve_config(conf)
        assert conf.value_get(PyAdcConf.ADC_CONF_BRD_STATE_MACHINE_STATUS) == adc_simple.ADC_CONF_100M14B4CHA_FSM_STATE_IDLE

    def test_adc_trg_sw_long_pre(self, adc_simple):
        """Test that the system triggers when we do the software
        trigger. We are careful here to see that the FSM goes into the
        PRE state where the software trigger should not work. We fire
        the software trigger anyway becase the driver will wait for
        the WAIT state"""
        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_BRD, 0)
        conf.mask_set(PyAdcConf.ADC_CONF_BRD_STATE_MACHINE_STATUS)
        adc_simple.retrieve_config(conf)
        assert conf.value_get(PyAdcConf.ADC_CONF_BRD_STATE_MACHINE_STATUS) == adc_simple.ADC_CONF_100M14B4CHA_FSM_STATE_IDLE

        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_ACQ, 0)
        conf.value_set(PyAdcConf.ADC_CONF_ACQ_PRE_SAMP, 100000)
        adc_simple.apply_config(conf, 0)
        adc_simple.acq_start(PyFmcAdc100m14b4ch.ADC_F_FLUSH,
                                 timeval(0,0))

        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_BRD, 0)
        conf.mask_set(PyAdcConf.ADC_CONF_BRD_STATE_MACHINE_STATUS)
        adc_simple.retrieve_config(conf)
        assert conf.value_get(PyAdcConf.ADC_CONF_BRD_STATE_MACHINE_STATUS) == adc_simple.ADC_CONF_100M14B4CHA_FSM_STATE_PRE

        adc_simple.trigger_fire()
        adc_simple.retrieve_config(conf)
        assert conf.value_get(PyAdcConf.ADC_CONF_BRD_STATE_MACHINE_STATUS) == adc_simple.ADC_CONF_100M14B4CHA_FSM_STATE_IDLE

class TestAdcTriggerTime(object):

    def test_adc_trg_tim_disable(self, adc_simple):
        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_TRG_TIM, 0)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_TIM_ENABLE, 0)
        adc_simple.apply_config(conf, 0)

        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_BRD, 0)
        conf.mask_set(PyAdcConf.ADC_CONF_UTC_TIMING_BASE_S)
        adc_simple.retrieve_config(conf)
        time_base = conf.value_get(PyAdcConf.ADC_CONF_UTC_TIMING_BASE_S)

        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_TRG_TIM, 0)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_TIM_SECONDS, time_base + 5)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_TIM_NANO_SECONDS, 0)
        adc_simple.apply_config(conf, 0)

        adc_simple.acq_start(PyFmcAdc100m14b4ch.ADC_F_FLUSH, timeval(0, 0))
        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(10,0))

    @pytest.mark.parametrize("seconds", [2, 3, 4, 5])
    @pytest.mark.parametrize("nanoseconds", [10**i for i in range(3, 9)])
    def test_adc_trg_tim(self, adc_simple, seconds, nanoseconds):
        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_BRD, 0)
        conf.mask_set(PyAdcConf.ADC_CONF_UTC_TIMING_BASE_S)
        adc_simple.retrieve_config(conf)
        time_base = conf.value_get(PyAdcConf.ADC_CONF_UTC_TIMING_BASE_S)

        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_TRG_TIM, 0)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_TIM_ENABLE, 0)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_TIM_SECONDS, time_base + seconds)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_TIM_NANO_SECONDS, nanoseconds)
        adc_simple.apply_config(conf, 0)

        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_TRG_TIM, 0)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_TIM_ENABLE, 1)
        adc_simple.apply_config(conf, 0)

        adc_simple.acq_start(PyFmcAdc100m14b4ch.ADC_F_FLUSH, timeval(0, 0))
        adc_simple.acq_poll(0, timeval(seconds + 1,0))
        buf_ptr = adc_simple.request_buffer(1024, None, 0)
        buf = buf_ptr.contents
        adc_simple.fill_buffer(buf_ptr, 0, timeval(10, 0))
        assert buf.tstamp.secs == time_base + seconds
        ticks_diff = abs(buf.tstamp.ticks - (nanoseconds * 125000000) / 1000000000)
        assert ticks_diff < 10 # 80ns margin
        assert buf.nsamples == 3
        adc_simple.release_buffer(buf_ptr, None)


class TestAdcTriggerThreshold(object):

    @pytest.mark.parametrize("channel", range(4))
    def test_adc_trg_thr_disable(self, adc_simple, channel):
        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_TRG_THR, channel)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_THRESHOLD, 0x1FFF)
        adc_simple.apply_config(conf, 0)

        buf = adc_simple.request_buffer(3, None, 0)  # adc_simple does 3

        adc_simple.pattern_data = 0
        adc_simple.acq_start(PyFmcAdc100m14b4ch.ADC_F_FLUSH, timeval(0, 0))
        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(1,0))

        adc_simple.pattern_data = 0x3FFF
        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(1,0))

        adc_simple.pattern_data = 0
        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(1,0))

    @pytest.mark.parametrize("channel", range(4))
    @pytest.mark.parametrize("threshold", [0xFFC, ])
    def test_adc_trg_thr_positive(self, adc_simple, channel, threshold):
        """The trigger threshold fire on positive slopes.
        Rembember that the ADC raw data is shifted by 2, so we use a
        threshold that does not use the last two bits"""
        min_value = 0
        adc_simple.pattern_data = min_value

        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_TRG_THR, channel)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_ENABLE, 1)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_POLARITY,
                       PyAdcAbstract.ADC_TRG_POL_POS)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_DELAY, 0)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_THRESHOLD, threshold)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_HYSTERESIS, 0)
        adc_simple.apply_config(conf, 0)

        adc_simple.acq_start(PyFmcAdc100m14b4ch.ADC_F_FLUSH, timeval(0, 0))

        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(2,0))
        adc_simple.pattern_data = (threshold >> 2) - 1
        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(2,0))
        adc_simple.pattern_data = min_value
        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(2,0))

        value = (threshold >> 2) + 1
        adc_simple.pattern_data = value
        adc_simple.acq_poll(0, timeval(0, 1000))
        buf = adc_simple.request_buffer(1024, None, 0)
        adc_simple.fill_buffer(buf, 0, timeval(10, 0))
        assert buf.contents.nsamples == 3
        assert buf.get_sample(channel, 0) == min_value, \
          buf.contents.data[:3*4]
        assert buf.get_sample(channel, 1) == value << 2, \
          buf.contents.data[:3*4]
        assert buf.get_sample(channel, 2) == value << 2, \
          buf.contents.data[:3*4]
        adc_simple.release_buffer(buf, None)

        adc_simple.acq_start(PyFmcAdc100m14b4ch.ADC_F_FLUSH, timeval(0, 0))
        adc_simple.pattern_data = min_value
        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(2,0))

    @pytest.mark.parametrize("channel", range(4))
    @pytest.mark.parametrize("threshold", [0xFFC, ])
    def test_adc_trg_thr_negative(self, adc_simple, channel, threshold):
        """The trigger threshold fire on negative slopes.
        Rembember that the ADC raw data is shifted by 2, so we use a
        threshold that does not use the last two bits"""
        max_value = 0x1FFF
        adc_simple.pattern_data = max_value

        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_TRG_THR, channel)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_ENABLE, 1)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_POLARITY,
                       PyAdcAbstract.ADC_TRG_POL_NEG)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_DELAY, 0)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_THRESHOLD, threshold)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_HYSTERESIS, 0)
        adc_simple.apply_config(conf, 0)

        adc_simple.acq_start(PyFmcAdc100m14b4ch.ADC_F_FLUSH, timeval(0, 0))

        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(2,0))
        adc_simple.pattern_data = (threshold >> 2) + 1
        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(2,0))
        adc_simple.pattern_data = max_value
        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(2,0))

        value = (threshold >> 2) - 1
        adc_simple.pattern_data = value
        adc_simple.acq_poll(0, timeval(0, 1000))
        buf = adc_simple.request_buffer(1024, None, 0)
        adc_simple.fill_buffer(buf, 0, timeval(10, 0))
        assert buf.contents.nsamples == 3
        assert buf.get_sample(channel, 0) == max_value << 2, \
          buf.contents.data[:3*4]
        assert buf.get_sample(channel, 1) == value << 2, \
          buf.contents.data[:3*4]
        assert buf.get_sample(channel, 2) == value << 2, \
          buf.contents.data[:3*4]
        adc_simple.release_buffer(buf, None)

        adc_simple.acq_start(PyFmcAdc100m14b4ch.ADC_F_FLUSH, timeval(0, 0))
        adc_simple.pattern_data = max_value
        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(2,0))

==> adc-lib-v2.6.0.tar: adc-lib-v2.6.0/pytest/fmc-adc-100m14b4cha/test_adc_trigger.py <==
"""
SPDX-License-Identifier: GPL-3.0-or-later
SPDX-FileCopyrightText: 2020 CERN
"""

import pytest
from PyAdcLib import PyAdcConf, PyFmcAdc100m14b4ch, timeval, PyAdcAbstract

class TestAdcTriggerSoftware(object):

    def test_adc_trg_sw(self, adc_simple):
        """Test that the system triggers when we do the software trigger"""
        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_BRD, 0)
        conf.mask_set(PyAdcConf.ADC_CONF_BRD_STATE_MACHINE_STATUS)
        adc_simple.retrieve_config(conf)
        assert conf.value_get(PyAdcConf.ADC_CONF_BRD_STATE_MACHINE_STATUS) == adc_simple.ADC_CONF_100M14B4CHA_FSM_STATE_IDLE
        adc_simple.acq_start(PyFmcAdc100m14b4ch.ADC_F_FLUSH,
                                 timeval(0,0))
        adc_simple.retrieve_config(conf)
        assert conf.value_get(PyAdcConf.ADC_CONF_BRD_STATE_MACHINE_STATUS) == adc_simple.ADC_CONF_100M14B4CHA_FSM_STATE_WAIT

        adc_simple.trigger_fire()
        adc_simple.retrieve_config(conf)
        assert conf.value_get(PyAdcConf.ADC_CONF_BRD_STATE_MACHINE_STATUS) == adc_simple.ADC_CONF_100M14B4CHA_FSM_STATE_IDLE

    def test_adc_trg_sw_long_pre(self, adc_simple):
        """Test that the system triggers when we do the software
        trigger. We are careful here to see that the FSM goes into the
        PRE state where the software trigger should not work. We fire
        the software trigger anyway becase the driver will wait for
        the WAIT state"""
        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_BRD, 0)
        conf.mask_set(PyAdcConf.ADC_CONF_BRD_STATE_MACHINE_STATUS)
        adc_simple.retrieve_config(conf)
        assert conf.value_get(PyAdcConf.ADC_CONF_BRD_STATE_MACHINE_STATUS) == adc_simple.ADC_CONF_100M14B4CHA_FSM_STATE_IDLE

        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_ACQ, 0)
        conf.value_set(PyAdcConf.ADC_CONF_ACQ_PRE_SAMP, 100000)
        adc_simple.apply_config(conf, 0)
        adc_simple.acq_start(PyFmcAdc100m14b4ch.ADC_F_FLUSH,
                                 timeval(0,0))

        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_BRD, 0)
        conf.mask_set(PyAdcConf.ADC_CONF_BRD_STATE_MACHINE_STATUS)
        adc_simple.retrieve_config(conf)
        assert conf.value_get(PyAdcConf.ADC_CONF_BRD_STATE_MACHINE_STATUS) == adc_simple.ADC_CONF_100M14B4CHA_FSM_STATE_PRE

        adc_simple.trigger_fire()
        adc_simple.retrieve_config(conf)
        assert conf.value_get(PyAdcConf.ADC_CONF_BRD_STATE_MACHINE_STATUS) == adc_simple.ADC_CONF_100M14B4CHA_FSM_STATE_IDLE

class TestAdcTriggerTime(object):

    def test_adc_trg_tim_disable(self, adc_simple):
        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_TRG_TIM, 0)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_TIM_ENABLE, 0)
        adc_simple.apply_config(conf, 0)

        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_BRD, 0)
        conf.mask_set(PyAdcConf.ADC_CONF_UTC_TIMING_BASE_S)
        adc_simple.retrieve_config(conf)
        time_base = conf.value_get(PyAdcConf.ADC_CONF_UTC_TIMING_BASE_S)

        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_TRG_TIM, 0)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_TIM_SECONDS, time_base + 5)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_TIM_NANO_SECONDS, 0)
        adc_simple.apply_config(conf, 0)

        adc_simple.acq_start(PyFmcAdc100m14b4ch.ADC_F_FLUSH, timeval(0, 0))
        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(10,0))

    @pytest.mark.parametrize("seconds", [2, 3, 4, 5])
    @pytest.mark.parametrize("nanoseconds", [10**i for i in range(3, 9)])
    def test_adc_trg_tim(self, adc_simple, seconds, nanoseconds):
        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_BRD, 0)
        conf.mask_set(PyAdcConf.ADC_CONF_UTC_TIMING_BASE_S)
        adc_simple.retrieve_config(conf)
        time_base = conf.value_get(PyAdcConf.ADC_CONF_UTC_TIMING_BASE_S)

        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_TRG_TIM, 0)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_TIM_ENABLE, 0)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_TIM_SECONDS, time_base + seconds)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_TIM_NANO_SECONDS, nanoseconds)
        adc_simple.apply_config(conf, 0)

        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_TRG_TIM, 0)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_TIM_ENABLE, 1)
        adc_simple.apply_config(conf, 0)

        adc_simple.acq_start(PyFmcAdc100m14b4ch.ADC_F_FLUSH, timeval(0, 0))
        adc_simple.acq_poll(0, timeval(seconds + 1,0))
        buf_ptr = adc_simple.request_buffer(1024, None, 0)
        buf = buf_ptr.contents
        adc_simple.fill_buffer(buf_ptr, 0, timeval(10, 0))
        assert buf.tstamp.secs == time_base + seconds
        ticks_diff = abs(buf.tstamp.ticks - (nanoseconds * 125000000) / 1000000000)
        assert ticks_diff < 10 # 80ns margin
        assert buf.nsamples == 3
        adc_simple.release_buffer(buf_ptr, None)


class TestAdcTriggerThreshold(object):

    @pytest.mark.parametrize("channel", range(4))
    def test_adc_trg_thr_disable(self, adc_simple, channel):
        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_TRG_THR, channel)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_THRESHOLD, 0x1FFF)
        adc_simple.apply_config(conf, 0)

        buf = adc_simple.request_buffer(3, None, 0)  # adc_simple does 3

        adc_simple.pattern_data = 0
        adc_simple.acq_start(PyFmcAdc100m14b4ch.ADC_F_FLUSH, timeval(0, 0))
        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(1,0))

        adc_simple.pattern_data = 0x3FFF
        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(1,0))

        adc_simple.pattern_data = 0
        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(1,0))

    @pytest.mark.parametrize("channel", range(4))
    @pytest.mark.parametrize("threshold", [0xFFC, ])
    def test_adc_trg_thr_positive(self, adc_simple, channel, threshold):
        """The trigger threshold fire on positive slopes.
        Rembember that the ADC raw data is shifted by 2, so we use a
        threshold that does not use the last two bits"""
        min_value = 0
        adc_simple.pattern_data = min_value

        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_TRG_THR, channel)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_ENABLE, 1)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_POLARITY,
                       PyAdcAbstract.ADC_TRG_POL_POS)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_DELAY, 0)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_THRESHOLD, threshold)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_HYSTERESIS, 0)
        adc_simple.apply_config(conf, 0)

        adc_simple.acq_start(PyFmcAdc100m14b4ch.ADC_F_FLUSH, timeval(0, 0))

        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(2,0))
        adc_simple.pattern_data = (threshold >> 2) - 1
        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(2,0))
        adc_simple.pattern_data = min_value
        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(2,0))

        value = (threshold >> 2) + 1
        adc_simple.pattern_data = value
        adc_simple.acq_poll(0, timeval(0, 1000))
        buf = adc_simple.request_buffer(1024, None, 0)
        adc_simple.fill_buffer(buf, 0, timeval(10, 0))
        assert buf.contents.nsamples == 3
        assert buf.contents.get_sample(channel, 0) == min_value, \
          buf.contents.data[:3*4]
        assert buf.contents.get_sample(channel, 1) == value << 2, \
          buf.contents.data[:3*4]
        assert buf.contents.get_sample(channel, 2) == value << 2, \
          buf.contents.data[:3*4]
        adc_simple.release_buffer(buf, None)

        adc_simple.acq_start(PyFmcAdc100m14b4ch.ADC_F_FLUSH, timeval(0, 0))
        adc_simple.pattern_data = min_value
        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(2,0))

    @pytest.mark.parametrize("channel", range(4))
    @pytest.mark.parametrize("threshold", [0xFFC, ])
    def test_adc_trg_thr_negative(self, adc_simple, channel, threshold):
        """The trigger threshold fire on negative slopes.
        Rembember that the ADC raw data is shifted by 2, so we use a
        threshold that does not use the last two bits"""
        max_value = 0x1FFF
        adc_simple.pattern_data = max_value

        conf = PyAdcConf(PyAdcConf.ADC_CONF_TYPE_TRG_THR, channel)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_ENABLE, 1)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_POLARITY,
                       PyAdcAbstract.ADC_TRG_POL_NEG)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_DELAY, 0)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_THRESHOLD, threshold)
        conf.value_set(PyAdcConf.ADC_CONF_TRG_THR_HYSTERESIS, 0)
        adc_simple.apply_config(conf, 0)

        adc_simple.acq_start(PyFmcAdc100m14b4ch.ADC_F_FLUSH, timeval(0, 0))

        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(2,0))
        adc_simple.pattern_data = (threshold >> 2) + 1
        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(2,0))
        adc_simple.pattern_data = max_value
        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(2,0))

        value = (threshold >> 2) - 1
        adc_simple.pattern_data = value
        adc_simple.acq_poll(0, timeval(0, 1000))
        buf = adc_simple.request_buffer(1024, None, 0)
        adc_simple.fill_buffer(buf, 0, timeval(10, 0))
        assert buf.contents.nsamples == 3
        assert buf.contents.get_sample(channel, 0) == max_value, \
          buf.contents.data[:3*4]
        assert buf.contents.get_sample(channel, 1) == value << 2, \
          buf.contents.data[:3*4]
        assert buf.contents.get_sample(channel, 2) == value << 2, \
          buf.contents.data[:3*4]
        adc_simple.release_buffer(buf, None)

        adc_simple.acq_start(PyFmcAdc100m14b4ch.ADC_F_FLUSH, timeval(0, 0))
        adc_simple.pattern_data = max_value
        with pytest.raises(OSError):
            adc_simple.acq_poll(0, timeval(2,0))
